package org.eso.ohs.core.dbb.examples;

import java.awt.Font;
import java.awt.GraphicsEnvironment;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.prefs.Preferences;

import javax.swing.AbstractAction;
import javax.swing.ButtonGroup;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.LookAndFeel;
import javax.swing.SwingUtilities;
import javax.swing.UIManager;
import javax.swing.WindowConstants;



/**
 * Class PreferencesFrame
 *
 * @version $Revision: 1.1 $
 * @author pnunes, Mar 2, 2005
 */

// $Id: PreferencesFrame.java,v 1.1 2005/03/04 12:39:11 pnunes Exp $ 
public class PreferencesFrame extends javax.swing.JFrame {
    
    private JPanel fontPanel;
    private JCheckBox jCheckBoxBold;
    private JCheckBox jCheckBoxItalic;
    private JComboBox jComboBoxFont;
    private JComboBox jComboBoxFontSize;
    private JLabel labelFont;
    private JLabel labelSize;    
    
    
    private JPanel lookAndFeelPanel;
    private  ButtonGroup lafGroup;
    
    
    private JPanel buttonPanel;
    private JButton buttonApply;
    private JButton buttonExit;
       
    
    //  Possible Look & Feels
    private static final String []  lafClasses = 
    { "com.sun.java.swing.plaf.mac.MacLookAndFeel",
      "javax.swing.plaf.metal.MetalLookAndFeel",
      "com.sun.java.swing.plaf.motif.MotifLookAndFeel",
      "com.sun.java.swing.plaf.windows.WindowsLookAndFeel",    
      "com.sun.java.swing.plaf.gtk.GTKLookAndFeel" };
    
    private static final String []  lafNames = 
    { "Mac",
      "Metal",
      "Motif",
      "Windows",    
      "GTK" };
    
    // 	The current Look & Feel
    private static String currentLookAndFeel = lafNames[1];
    
    /** Creates new form PreferencesFrame */
    public PreferencesFrame() {
        initComponents();
        loadPreferences();
    }
    
    /** 
     * Initializes the components
     * 
     */
    private void initComponents() {
        
        setDefaultCloseOperation(WindowConstants.EXIT_ON_CLOSE);
        
        /* font panel components*/
        fontPanel = new JPanel();
        
        labelFont = new JLabel(); 
        labelFont.setText("Font:");
        
        labelSize = new JLabel();
        labelSize.setText("Size:");      
        
        initializeFonts();
        
        
        jCheckBoxBold = new JCheckBox();
        jCheckBoxBold.setText("Bold");
        
        jCheckBoxItalic = new JCheckBox();
        jCheckBoxItalic.setText("Italic");
        
        fontPanel.add(labelFont);
        fontPanel.add(jComboBoxFont);
        fontPanel.add(labelSize);
        fontPanel.add(jComboBoxFontSize);       
        fontPanel.add(jCheckBoxBold);        
        fontPanel.add(jCheckBoxItalic);

        
        /* look and feel panel components */
        lookAndFeelPanel = new JPanel();
        createLafRadioButtons();

        
        /* buttons panel */
        buttonPanel = new JPanel();
        buttonApply = new JButton("Save/Apply");
        buttonApply.addActionListener(new ApplyActionListener());
        
        buttonExit = new JButton("Exit");
        buttonExit.addActionListener(new ExitActionListener());
        buttonPanel.add(buttonApply);
        buttonPanel.add(buttonExit);
        
                
        getContentPane().add(fontPanel, java.awt.BorderLayout.NORTH);
        getContentPane().add(lookAndFeelPanel, java.awt.BorderLayout.CENTER);
        getContentPane().add(this.buttonPanel, java.awt.BorderLayout.SOUTH);
        pack();
    }
    
    public void createLafRadioButtons() {
    
       lafGroup = new ButtonGroup();
       
       for (int i = 0; i < lafClasses.length; i++) {
           String laf = lafClasses[i];
           String lafName = lafNames[i];
           boolean selected = false;
           
           if (isAvailableLookAndFeel(laf)) {
               if (lafName.equals(currentLookAndFeel))
                   selected = true;
               JRadioButton button = new JRadioButton(lafName,selected);
               button.addActionListener(new ChangeLookAndFeelAction(this, laf));
               
               lafGroup.add(button);
               lookAndFeelPanel.add(button);
           }
       }
       
       
       
       //new ChangeLookAndFeelAction(this, laf);
        
        
    }
    
    protected boolean isAvailableLookAndFeel(String laf) {
        try { 
            Class lnfClass = Class.forName(laf);
            LookAndFeel newLAF = (LookAndFeel)(lnfClass.newInstance());
            return newLAF.isSupportedLookAndFeel();
        } catch(Exception e) { // If ANYTHING weird happens, return false
            return false;
        }
    }
    
    private void initializeFonts() {
        GraphicsEnvironment ge = GraphicsEnvironment.getLocalGraphicsEnvironment();        
        String [] fontNames = ge.getAvailableFontFamilyNames();
        String [] fontSizes = new String [] {"8", "9", "10", "11", "12", "14", "16", "18", "20", "22", "24", "26", "28", "36", "48", "72" };         
        jComboBoxFont = new JComboBox(fontNames);        
        jComboBoxFontSize = new JComboBox(fontSizes);               
    }
    
    
    public void loadPreferences() {
        Preferences prefs = Preferences.userNodeForPackage(this.getClass());
        
        // get the preferences
        String font = prefs.get("font","Arial");
        int fontSize = prefs.getInt("font-size",10) ;        
        boolean bold = prefs.getBoolean("bold",false);
        boolean italic = prefs.getBoolean("italic",false);
        String laf = prefs.get("laf",currentLookAndFeel);
         
        
        // updates combo boxes and check boxes
        for (int i = 0; i < jComboBoxFont.getItemCount()  ; i++) {
            String fnt = (String)jComboBoxFont.getItemAt(i);
            if (fnt.equals(font)) {
                jComboBoxFont.setSelectedIndex(i);
                break;
            }            
        }
        
        for (int i = 0; i < jComboBoxFontSize.getItemCount()  ; i++) {
            String size = (String)jComboBoxFontSize.getItemAt(i);
            if (size.equals(fontSize+"")) {
                jComboBoxFontSize.setSelectedIndex(i);
                break;
            }            
        }
        
        jCheckBoxBold.setSelected(bold);
        jCheckBoxItalic.setSelected(italic);
        
                        
        //   sets the frames fonts
        int fontStyle;
        
        if (bold && italic) {
            fontStyle = 3;            
        } else {
            if (bold) 
                fontStyle = Font.BOLD;
            else
                if (italic)
                    fontStyle = Font.ITALIC;
                else
                    fontStyle = Font.PLAIN;
                
        }
        applyFonts(new Font(font,fontStyle,fontSize));
        
        // look and feel
        setLookAndFeel(laf);
                       
    }
    
    /**
     * Stores the current L&F, and calls updateLookAndFeel, below
     */
    public void setLookAndFeel(String laf) {
        if( currentLookAndFeel != laf ) {
            currentLookAndFeel = laf;            
            updateLookAndFeel();
        }
    }
    
    
    /**
     * Sets the current L&F on each demo module
     */
    public void updateLookAndFeel() {
        try {
            UIManager.setLookAndFeel(currentLookAndFeel);            
            // 	update LAF for the toplevel frame, too            
            SwingUtilities.updateComponentTreeUI(this);
            

        } catch (Exception ex) {
            System.out.println("Failed loading L&F: " + currentLookAndFeel);
            System.out.println(ex);
        }
    }

    
    protected void applyFonts( Font f ) {
        
        labelFont.setFont(f);
        labelSize.setFont(f);
        jCheckBoxBold.setFont(f);
        jCheckBoxItalic.setFont(f);
        buttonApply.setFont(f);
        buttonExit.setFont(f);
        jComboBoxFont.setFont(f);
        jComboBoxFontSize.setFont(f);
        
        for (int i = 0; i < lookAndFeelPanel.getComponents().length; i++) {
            lookAndFeelPanel.getComponent(i).setFont(f);
        }
        
        
        repaint();  
        pack();
    }
    
    public void savePreferences() {
        String font = (String)jComboBoxFont.getSelectedItem();
        int fontSize = new Integer( (String)jComboBoxFontSize.getSelectedItem() ).intValue();
        boolean bold = jCheckBoxBold.isSelected();
        boolean italic = jCheckBoxItalic.isSelected();
        
        
        Preferences prefs = Preferences.userNodeForPackage(this.getClass());
        prefs.put("font",font);
        prefs.putInt("font-size",fontSize);
        prefs.putBoolean("bold",bold);
        prefs.putBoolean("italic",italic);
        prefs.put("laf",currentLookAndFeel);
        
        loadPreferences();
        
    }
    
    
    
    
    protected class ApplyActionListener implements ActionListener {
        
            public void actionPerformed(ActionEvent ae) {
                savePreferences();                               
            }                
    }
    
    protected class ExitActionListener implements ActionListener {
        
            public void actionPerformed(ActionEvent ae) {                
                System.exit(0);
            }                
    }
    
    protected class ChangeLookAndFeelAction extends AbstractAction {
        PreferencesFrame frame;
        String laf;
        protected ChangeLookAndFeelAction(PreferencesFrame frame, String laf) {
            super("ChangeTheme");
            this.frame = frame;
            this.laf = laf;
        }

        public void actionPerformed(ActionEvent e) {
            frame.setLookAndFeel(laf);
            frame.pack();
        }
    }
     
    
    
   
    public static void main( String []  args ) {
        
        PreferencesFrame prefsFrame = new PreferencesFrame();
        prefsFrame.setVisible(true);
        
        
        
    }

    
    
}






