/* Created on Dec 9, 2003 */

package org.eso.ohs.core.dbb.htmlclient;

import java.net.MalformedURLException;
import java.rmi.NotBoundException;
import java.rmi.RemoteException;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import javax.servlet.http.HttpServletRequest;

import org.eso.ohs.core.dbb.server.DbbQuery;
import org.eso.ohs.core.dbb.server.DbbSqlEngine;
import org.eso.ohs.core.dbb.sql.DbbSqlChunk;
import org.eso.ohs.core.dbb.sql.DbbSqlQueryChunks;

/**
 * TODO: javadoc
 * 
 * @author amchavan
 * @version $Revision: 1.1 $
 */

// $Id: DbbHtmlView.java,v 1.1 2004/02/17 11:16:25 tcanavan Exp $

public class DbbHtmlView {

	
	public static final String cvsID_ =
	"$Id: DbbHtmlView.java,v 1.1 2004/02/17 11:16:25 tcanavan Exp $";

	/** Width of the table borders */
	private static final int TABLEBORDER = 0;

	/** Number of displayed columns */
	private int cols = -1;

	/**
	 * DbbPanel for selecting which columns will be displayed
	 */
	protected DbbHtmlPanel columnsPanel_ = null;

	/** List of panels added "North" in the view. */
	private List north = null;

	/** List of panels added "South" in the view. */
	private List south = null;

	/** List of panels added "East" in the view. */
	private List east = null;

	/** List of panels added "West" in the view. */
	private List west = null;

	/**
	 * DbbHtmlPanel for the widgets that will generate the "ORDER BY" clauses
	 */
	protected DbbHtmlPanel sortPanel_ = null;
	
	/** SQL engine associated to this view. */
	private DbbSqlEngine sqlEngine_;

	/**
	 * DbbHtmlPanel for selecting which rows will be retrieved; i.e., the
	 * widgets that will generate the "WHERE" clauses.
	 */
	protected DbbHtmlPanel wherePanel_ = null;
	
	/**
	 * Build an HTML database browser view
	 *
	 * @param sqlEngine Reference to the SQL engine which does all the work
	 */
	public DbbHtmlView( DbbSqlEngine sqlEngine ) {
		sqlEngine_ = sqlEngine;
		
		north = new ArrayList();
		south = new ArrayList();
		east  = new ArrayList();
		west  = new ArrayList();
	}

	/**
	 * @return The columns selection panel
	 */
	public DbbHtmlPanel getColumnsPanel() {
		return columnsPanel_;
	}

	/**
	 * @return The sort panel
	 */
	public DbbHtmlPanel getSortPanel() {
		return sortPanel_;
	}

	/**
	 * @return The rows selection panel
	 */
	public DbbHtmlPanel getWherePanel() {
		return wherePanel_;
	}

	/**
	 * Generate a fragment of HTML code rendering this view. The fragment is
	 * suitable to be included in the HTML rendering of an HTML form.
	 * <BR>
	 * The generated fragment is not indented.
	 * 
	 * @return An HTML fragment rendering this widget.
	 */
	public String render() {
		return render( 0 );
	}

	/**
	 * Generate a fragment of HTML code rendering this panel. The fragment is
	 * suitable to be included in the HTML rendering of an HTML form.
	 * 
	 * @param indentLevel
	 *            The level of indentation for the HTML fragment. Each level is
	 *            indented by INDENTSPACES spaces.
	 * 
	 * @return An HTML fragment rendering this widget.
	 * 
	 * @see DbbHtmlWidget#INDENTSPACES
	 */
	public String render( int indentLevel ) {
		
		
		String ind0 = DbbHtmlWidget.indentSpaces( indentLevel     );
		String ind1 = DbbHtmlWidget.indentSpaces( indentLevel + 1 );
		String ind2 = DbbHtmlWidget.indentSpaces( indentLevel + 2 );
		String ind3 = DbbHtmlWidget.indentSpaces( indentLevel + 3 );
		String ind4 = DbbHtmlWidget.indentSpaces( indentLevel + 4 );
		
		StringBuffer sb = new StringBuffer();
		Iterator panels = null;
		
		sb.append( ind0 )
			.append( "<table border=\"" ).append(TABLEBORDER).append("\">\n" );
		
		// North
		//---------------------------------------
		sb.append( ind1 ).append( "<tr> <!-- North -->\n" );		
		panels = north.iterator();
		while ( panels.hasNext()) {
			DbbHtmlPanel panel = (DbbHtmlPanel) panels.next();
			
			sb.append( ind2 ).append( "<td colspan=\"2\">\n" );
			sb.append( panel.render( indentLevel + 3 ));
			sb.append( ind2 ).append( "</td>\n" );
		}
		sb.append( ind1 ).append( "</tr> <!-- North -->\n" );
		
		
		sb.append( ind1 ).append( "<tr> <!-- East & West -->\n" );
		
		// West
		//---------------------------------------
		sb.append( ind2 ).append( "<td> <!-- West -->\n" );
		sb.append( ind3 )
			.append( "<table border=\"" ).append(TABLEBORDER).append("\">\n" );
		panels = west.iterator();
		while ( panels.hasNext()) {
			DbbHtmlPanel panel = (DbbHtmlPanel) panels.next();
			sb.append( ind4 ).append( "<tr><td>\n" );
			sb.append( panel.render(indentLevel + 5 ));
			sb.append( ind4 ).append( "</td></tr>\n" );
		}
		sb.append( ind3 ).append( "</table>\n" );
		sb.append( ind2 ).append( "</td> <!-- West -->\n" );

		// East
		//---------------------------------------
		sb.append( ind2 ).append( "<td> <!-- East -->\n" );
		sb.append( ind3 )
			.append( "<table border=\"" ).append(TABLEBORDER).append("\">\n" );
		panels = east.iterator();
		while ( panels.hasNext()) {
			DbbHtmlPanel panel = (DbbHtmlPanel) panels.next();
			sb.append( ind4 ).append( "<tr><td>\n" );
			sb.append( panel.render(indentLevel + 5 ));
			sb.append( ind4 ).append( "</td></tr>\n" );
		}
		sb.append( ind3 ).append( "</table>\n" );
		sb.append( ind2 ).append( "</td> <!-- East -->\n" );
		
		
		sb.append( ind1 ).append( "</tr> <!-- East & West -->\n" );	
		
		// South
		//---------------------------------------
		
		sb.append( ind1 ).append( "<tr> <!-- South -->\n" );		
		panels = south.iterator();
		while ( panels.hasNext()) {
			DbbHtmlPanel panel = (DbbHtmlPanel) panels.next();
			
			sb.append( ind2 ).append( "<td colspan=\"2\">\n" );
			sb.append( panel.render( indentLevel + 3 ));
			sb.append( ind2 ).append( "</td>\n" );
		}
		sb.append( ind1 ).append( "</tr> <!-- South -->\n" );
		
		
		sb.append( ind0 ).append( "</table>\n" );
		
		return sb.toString();
	}
	
	/**
	 * Set a column selection panel
	 *
	 * @param panel     DbbHtmlPanel defing a column selection panel
	 * @param position  Relative position in the view (NSEW)
	 */
	public void setColumnsPanel(DbbHtmlPanel panel, String position) {
		setColumnsPanel(panel, position, true, -1, -1);
	}

	/**
	 * Set a column selection panel
	 *
	 * @param panel     DbbHtmlPanel defing a column selection panel
	 * @param position  Relative position in the view (NSEW)
	 * @param visible   specifies if the panel is visible
	 */
	public void setColumnsPanel(DbbHtmlPanel panel, String position, boolean visible) {
		setColumnsPanel(panel, position, visible, -1, -1);
	}

	/**
	 * Set a column selection panel
	 *
	 * @param panel     DbbHtmlPanel defing a column selection panel
	 * @param position  Relative position in the view (NSEW)
	 * @param visible   specifies if the panel is visible
	 * @param x         Ignored
	 * @param y         Ignored
	 */
	public void setColumnsPanel( DbbHtmlPanel panel,
			                     String position,
								 boolean visible,
								 int x,
								 int y ) {
		columnsPanel_ = panel;
		addPanel( panel, position, visible );
	}
	
	
	/**
	 * Set a sort panel (delegated to define the sorting criteria)
	 * @param panel     DbbHtmlPanel defing a sort panel
	 * @param position  Relative position in the view (NSEW)
	 */
	public void setSortPanel(DbbHtmlPanel panel, String position) {
		setSortPanel(panel, position, true, -1, -1);
	}

	/**
	 * Set a sort panel (delegated to define the sorting criteria)
	 * @param panel     DbbHtmlPanel defing a sort panel
	 * @param position  Relative position in the view (NSEW)
	 * @param visible   specifies if the panel is visible
	 */
	public void setSortPanel(DbbHtmlPanel panel, String position, boolean visible) {
		setSortPanel(panel, position, visible, -1, -1);
	}

	/**
	 * Set a sort panel (delegated to define the sorting criteria)
	 * @param panel     DbbHtmlPanel defing a sort panel
	 * @param position  Relative position in the view (NSEW)
	 * @param visible   specifies if the panel is visible
	 * @param x         Ignored
	 * @param y         Ignored
	 */
	public void setSortPanel( DbbHtmlPanel panel,
							  String position,
							  boolean visible,
							  int x,
							  int y ) {
		sortPanel_ = panel;
		addPanel( panel, position, visible );
	}

	/**
	 * Set a row selection panel (delegated to define a where clause)
	 *
	 * @param panel     DbbHtmlPanel defing a row selection panel
	 * @param position  Relative position in the view (NSEW)
	 */
	public void setWherePanel(DbbHtmlPanel panel, String position) {
		setWherePanel(panel, position, true, -1, -1);
	}

	/**
	 * Set a row selection panel (delegated to define a where clause)
	 *
	 * @param panel     DbbHtmlPanel defing a row selection panel
	 * @param position  Relative position in the view (NSEW)
	 * @param visible   specifies if the panel is visible
	 */
	public void setWherePanel(DbbHtmlPanel panel, String position, boolean visible) {
		setWherePanel(panel, position, visible, -1, -1);
	}

	/**
	 * Set a row selection panel (delegated to define a where clause)
	 *
	 * @param panel     DbbHtmlPanel defing a row selection panel
	 * @param position  Relative position in the view (NSEW)
	 * @param visible   specifies if the panel is visible
	 * @param x         Ignored
	 * @param y         Ignored
	 */
	public void setWherePanel( DbbHtmlPanel panel,
			                   String position,
							   boolean visible,
							   int x,
							   int y ) {
		wherePanel_ = panel;
		addPanel( panel, position, visible );
	}

	/**
	 * @param panel     A DbbHtmlPanel to add to this view
	 * @param position  Relative position in the view (NSEW)
	 * @param visible   Flag: specifies if the panel is visible
	 */
	private void addPanel( DbbHtmlPanel panel, String position, boolean visible) {
		
		if( ! visible )
			return;
		String p = position.toUpperCase().substring(0,1);
		if(      p.equals( "N" ))
			north.add( panel );
		else if( p.equals( "S" ))
			south.add( panel );
		else if( p.equals( "E" ))
			east.add( panel );
		else if( p.equals( "W" ))
			west.add( panel );
		else
			throw new IllegalArgumentException(
				"position=\"" + position + "\", should be one of NSEW." );  
    }
    
    /**
     * Collect all the chunks from the widgets, pass them on to the
     * SQL engine, and have it return the SQLquery text.
     *
     * @return Sql statement
     */
    protected DbbSqlQueryChunks getQueryChunks( HttpServletRequest req ) {

        DbbSqlQueryChunks query_chunks = new DbbSqlQueryChunks();
        DbbSqlChunk t[];
        
        if (columnsPanel_ != null) {
            t = columnsPanel_.getActiveChunks( req );
//            for( int i = 0; i < t.length; i++ )
//                System.out.println( ">>> columns: t[i]: " + t[i] );
            query_chunks.columns = t;
            //query_chunks.appendRows(columnsPanel_.getAdditionalChunks());
        }

        if (wherePanel_ != null) {
            t = wherePanel_.getActiveChunks( req );
//            for( int i = 0; i < t.length; i++ )
//                System.out.println( ">>> where: t[i]: " + t[i] );
               
            query_chunks.appendRows( t );
            //query_chunks.appendRows(wherePanel_.getAdditionalChunks());
        }

        if (sortPanel_ != null) {
            t = sortPanel_.getActiveChunks( req );
//            for( int i = 0; i < t.length; i++ )
//                System.out.println( ">>> sort: t[i]: " + t[i] );
            query_chunks.sort = t;
        }
        
        /* Send the chunks to the sql engine to build the query. */
        return query_chunks;
    }

    /**
     * Method runQuery start the query from the engine. 
     * Used by DBIOThread
     * @return DbbQuery
     * @throws RemoteException
     * @throws SQLException
     * @throws MalformedURLException
     * @throws NotBoundException
     */
    public DbbQuery runQuery( HttpServletRequest req )
        throws
            RemoteException,
            SQLException,
            MalformedURLException,
            NotBoundException {
        return sqlEngine_.startQuery( getQueryChunks( req ));
    }

}
