package org.eso.ohs.core.dbb.server;

// $Id: DbbQuery.java,v 1.1 2004/02/17 11:16:24 tcanavan Exp $

import java.rmi.RemoteException;
import java.sql.SQLException;

import org.eso.ohs.core.dbb.sql.DbbSqlQueryResult;

/**   
 * This is the main engine of a database browser: builds SQL
 * statements, performs queries (within threads), returns result data.
 *
 * @author Bob Kemp
 * <br>ChangeLog: C. Boarotto, ESO 10-Sep-2000
 *
 */
public interface DbbQuery {

    /** 
     * Set the number of rows to be displayed per time. 
     *
     * @param n Number of rows to be displayed.
     * @exception RemoteException remote exception occurs on app
     *		  server
     */
    public void setNumOfRowsToGet(int n)
	throws RemoteException;


    /** 
     * Used internally by the view in order to get the result data
     * structure.
     *
     * @return DbbSqlQueryResult data structure containing the query
     *	       result.
     * @exception RemoteException remote exception occurs on app
     *		  server
     */
    public DbbSqlQueryResult getResult()
	throws RemoteException;


    /** 
     * Has the result set any more rows to retrieve.
     *
     * @return boolean if numRows > 0
     * @exception RemoteException remote exception occurs on app
     *		  server
     */
    public boolean hasMoreRows()
	throws RemoteException;


    /** 
     * Read rows from the result set.   
     *
     * @exception RemoteException remote exception occurs on app
     *		  server
     * @exception SQLException JDBC Exception
     */
    public void readRows()
	throws SQLException, RemoteException;


    /** 
     * Close the result set relaease the database connection 
     *
     * @exception RemoteException remote exception occurs on app
     *		  server
     * @exception SQLException JDBC Exception
     */
    public void close()
	throws SQLException, RemoteException;
    
    
	/**
	 * Method getEmptyResult.
	 * @return DbbSqlQueryResult
	 */
	public DbbSqlQueryResult getEmptyResult()
    throws SQLException, RemoteException;

	/**
	 * Method appendRows.
	 * @param result
	 * @return DbbSqlQueryResult
	 */
	public DbbSqlQueryResult appendRows(DbbSqlQueryResult result)
    throws SQLException, RemoteException;

}



