package org.eso.ohs.core.gui.models;

import javax.swing.text.AttributeSet;
import javax.swing.text.BadLocationException;

/**
 * A Document model to only permit the input of a single integer.
 * 
 * <p>
 * $Id: XdegModel.java,v 1.1 2004/02/17 10:35:21 tcanavan Exp $
 */

public class XdegModel extends InputMaskModel {

    private final int X_MAX = 360;
    private final float X_MIN = -360;

    private final static String legalValuesPattern_ = "(\\d{0,4})?([-]?\\d*)";

    /**
     * Constructor for the model.
     */
    public XdegModel() {
        super(legalValuesPattern_);
    }

    /**
     * Refuse input that is above or below the limits of the
     * textfield.
     */
    public void insertString(int offset, String str, AttributeSet a)
        throws BadLocationException {

        //stdlog_.debug("String: <"+str+">");
        super.insertString(offset, str, a);
        //stdlog_.debug("buffer: <"+buffer_+">");
        try {
            String buffer_val = buffer_.toString();

            Float f = Float.valueOf(buffer_val);
            if (f.floatValue() < X_MIN) {
                remove(offset, str.length());
            } else if (f.floatValue() > X_MAX) {
                remove(offset, str.length());
            }

        } catch (NumberFormatException e) {
            // nothing, the string is not complete...
        }
    }
}
