package org.eso.ohs.core.utilities;

/**
 * A generic interface to a tree node for use with the GTreeModel interface.
 * The interface is intended to be flexible enough to allow easy implementation
 * and application (e.g for use in Config information handling with JDOM).
 *
 * Please see GTreeModel for other details.
 *
 * @author Nick Kornweibel for ESO.
 */

import java.util.Enumeration;

public interface GTreeNode {

    /**
     * Set the parent of this to node.
     */
    public void setParent(GTreeNode node);

    /**
     * @return the parent of this node, or null if there is not one.
     */
    public GTreeNode getParent();

    /**
     * @return the complete path from the root node to the current one.
     */
    public String getPathName();
  /**
     * @return the complete path from the root node to the parent one.
     */
    public String getPathParentName();

    /**
     * @return true if node is a leaf, else false.
     */
    public boolean isLeaf();
   /**
     * @return true if node is the root, else false.
     */
    public boolean isRoot();

    /**
     * @return the name of this node.
     */
    public String getName();

    /**
     * Set the name of the node.
     */
    public void setName(String name);

    /**
     * @return the value object of this node.
     * Useful when a node value object is not a String.
     */
    public Object get();

    /**
     * Set the value object
     */
    public void set(Object val);

    /**
     * Set the val object as a string
     */
    public void setValue(String val);

    /**
     * @return the value of value object toString() method.
     * Used in the majority of cases as the value is a String.
     */
    public String getValue();

    /**
     * @return the child of node with the name childName,
     *     or null if not found.
     */
    public GTreeNode getChild(String childName);

    /**
     * @return the child of node with the name child.getName(),
     * or null if not found.
     */
    public GTreeNode getChild(GTreeNode child);

    /**
     * @return the number of children of node.
     */
    public int getChildCount();

    /**
     * @return an Enumeration of node's children.
     */
    public Enumeration getChildren();

    /**
     * Add child as a child of aNode (no checking for duplicates).
     */
    public void addChild(GTreeNode child);

    /**
     * @return (and remove) child from aNode, or null if not found.
     */
    public GTreeNode removeChild(GTreeNode child);

    /**
     * @return (and remove) the first child of aNode with the name childName.
     */
    public GTreeNode removeChild(String childName);
}
