/* $Id: Affiliation.java,v 1.19 2005/11/09 15:48:16 ddorigo Exp $ */

package org.eso.ohs.dfs;

/**
 * Affiliation business object class. This class is dependent from
 * EsoUser, therefore the object is retrieved from DbaseHandlerEsoUser
 * <br>
 * 
 * @see org.eso.ohs.persistence.dbase.phase1.DbaseHandlerEsoUser 
 * @author D. Dorigo, 10-Mar-2000
 */
public class Affiliation extends BusinessObject implements Comparable {

	/**
     * 
     */
    private static final long serialVersionUID = 1L;
    public static long DEFAULT_ACCESSED_TIME = 0;
	private long accessedLetter_ = 0;
	private static final int MAX_VARCHAR = 255;
	private long userId_;

	private int period_ = 0;

	private int cycleId_ = 0;

	private String cycleLabel_ = "";

	/** 
	 * country membership 
	 */
	private String country_code_ = "";

	/**
	 * User's affiliation (may contain latex formattings). 
	 */
	private String institution_ = "";

	//----------------------------------------
	// Constructor
	//----------------------------------------

	/**
	 * Create an Affiliation .    
	 */
	public Affiliation() {
		super();
	}

	/**
	   * Create an Affiliation. 
	   * @param  id user id for this afiliation
	   * @param  period 
	   * @param  cycleId 
	   * @param country_code the country to which the user belong for this cycle 
	   * @param  institution 
	   */
	public Affiliation(
		long id,
		Cycle cycle,
		String country_code,
		String institution,
		long accessedLetter) {
		super();
		setId(id);
		setUserId(id);
		setPeriod(cycle.getPeriod());
		setCycleId(cycle.getCycleId());
		setCycleLabel(cycle.getLabel());
		setCountryCode(country_code);
		setInstitution(institution);
		setAccessedLetter(accessedLetter);
	}

	/**
	   * Create an Affiliation. 
	   * @param  id user id for this afiliation
	   * @param  period 
	   * @param  cycleId 
	   * @param country_code the country to which the user belong for this cycle 
	   * @param  institution 
	   */
	public Affiliation(
		long id,
		int period,
		int cycleId,
		String country_code,
		String institution,
		long accessedLetter) {
		super();
		setId(id);
		setUserId(id);
		setPeriod(period);
		setCycleId(cycleId);
		setCountryCode(country_code);
		setInstitution(institution);
		setAccessedLetter(accessedLetter);
	}

	/**
	 * @param id
	 */
	public void setUserId(long id) {
		userId_ = id;

	};

	/**
	 * @param id
	 */
	public long getUserId() {
		return userId_;

	};

	//----------------------------------------
	// Public Methods
	//----------------------------------------

	/**
	 * Get value for period_ property
	 *
	 * @return the current value of the property
	 */
	public int getPeriod() {
		return period_;
	}

	/**
	 * Set new value for period_ property
	 *
	 * @param newValue the new value for the property
	 */
	public void setPeriod(int newValue) {
		period_ = newValue;
		firePropertyChangeEvent();
	}

	/**
	 * Get value for cycleId_ property
	 *
	 * @return the current value of the property
	 */
	public int getCycleId() {
		return cycleId_;
	}

	/**
	  * Get value
	  *
	  * @return the current value of the property
	  */
	public String getCycleLabel() {

		return cycleLabel_;
	}

	/**
	 * Set new value
	 *
	 * @param newValue the new value for the property
	 */
	public void setCycleLabel(String newValue) {
		cycleLabel_ = newValue;
	}
	/**
	 * Set new value for cycleId_ property
	 *
	 * @param newValue the new value for the property
	 */
	public void setCycleId(int newValue) {
		cycleId_ = newValue;
		firePropertyChangeEvent();
	}

	/**
	 * Get value for country_code_ property
	 *
	 * @return the current value of the property
	 */
	public String getCountryCode() {
		return country_code_;
	}

	/**
	 * Set new value for country_code_ property
	 *
	 * @param newValue the new value for the property
	 */
	public void setCountryCode(String newValue) {
		int maxLength = MAX_VARCHAR;
		country_code_ = getSubString(newValue, maxLength);
		firePropertyChangeEvent();
	}

	private String getSubString(String value, int maxLength) {
		if (value == null) {
			value = "";
		}

		if (value.length() <= maxLength) {
			maxLength = value.length();
		}
		return value.substring(0, maxLength);
	}
	/**
	 * Get value for affiliation_ property
	 *
	 * @return the current value of the property
	 */
	public String getInstitution() {
		return institution_;
	}

	/**
	 * Set new value for affiliation_ property
	 *
	 * @param newValue the new value for the property
	 */
	public void setInstitution(String newValue) {
		int maxLength = MAX_VARCHAR;
		institution_ = getSubString(newValue, maxLength);
		firePropertyChangeEvent();
	}

	public Object accept(BusinessVisitor v, Object arg) {
		return null;
	}

	/**
	 * Check if a given eso user object is equal to the current one
	 * just comparig the database ids.  
	 */
	public boolean equals(Object obj) {

		boolean ret = false;
		boolean retId = false;
		boolean retPeriod = false;
		boolean retCycle = false;

		try {
			Affiliation user = (Affiliation) obj;

			retId = (user.getId() == this.getId());
			retPeriod = (user.getPeriod() == this.getPeriod());
			retCycle = (user.getCycleId() == this.getCycleId());
			ret = (retId && retPeriod && retCycle);
		} catch (Exception e) {
			ret = false;
			// debug
			e.printStackTrace();
		}

		return ret;
	}
	/**
	 * @see org.eso.ohs.dfs.StorableObject#postProecess()
	 */
	public void postProecess() {
	}

	/**
	 * @return time
	 */
	public long getAccessedLetter() {
		return accessedLetter_;
	}

	/**
	 * @param l
	 */
	public void setAccessedLetter(long l) {
		accessedLetter_ = l;
	}

	/* (non-Javadoc)
	 * @see java.lang.Comparable#compareTo(java.lang.Object)
	 */
	public int compareTo(Object obj) {
		int result;
		if (obj instanceof Affiliation) {
			Affiliation affil = (Affiliation) obj;
			result = (int) (getCycleId() - affil.getCycleId());
		} else {
			throw new ClassCastException("Not an affiliation obj:" + obj);
		}
		return result;
	}

}
