package org.eso.ohs.dfs.docviewdatatrans;



import org.eso.ohs.core.utilities.Convert;

/**
 * A transducer to convert set/get of String values into set/get
 * on an int property.  The target property's name is given to the
 * constructor.
 *
 * For example, users will call it's
 * setValue/getValue methods.  Those calls will be translated into
 * calls to setX/getX on the object given to the constructor.
 *
 *
 * @author  cmavros 13/Sept/01
 */
public class ExecutionTimeAdaptor extends DegreeAdaptor {

    /**
     * Construct a transducer.
     */
    public ExecutionTimeAdaptor() {
	super();
    }

    /**
     * Construct a transducer to convert set/get of String values into
     * setX/getX on an int property of <code>object</code>.
     */
    public ExecutionTimeAdaptor(Object obj, String fieldName) {
	super(obj, fieldName);
    }

    /**
     * Converts data from a String to a meta-level equivalent of the
     * actual property's type, i.e. to a java.lang.Integer
     * representing an int.  Used with reflection to set
     * the new property value.
     * 
     * <p>
     * Execution time values are stored in seconds,
     * even though they are displayed to the user as hours:minutes:seconds.
     * 
     * @param value the new value of the property.
     */
    public Object toMetaLevelObject(String value) {
	int milliArcSecondsOfHours = convertToMilliArcSeconds(value);
	return new Integer( milliArcSecondsOfHours);
    }

    /**
     * Converts data to a String from the meta-level equivalent of the
     * actual property's type, i.e. from a java.lang.Long
     * to a long int.  Used with reflection to get
     * the current property value.
     *
     * <p>
     * Execution time values are stored in seconds,
     * even though they are displayed to the user as hours:minutes:seconds.
     * 
     * @param value	value of the property.
     */
    public String fromMetaLevelObject(Object obj) {
	if (obj == null) {
	    return Convert.EXECTIME_ERR_STRING;
	}

        int value = ((Integer)obj).intValue();
	if (value < 0 || value > Convert.EXECTIME_MAX) {
	    return Convert.EXECTIME_ERR_STRING;
	}
        return convertToHMS(value);
    }

    /**
     */
    protected int convertToMilliArcSeconds( String value ) {
	return( Convert.HHMMSSToSec( value ));
    }

    /**
     */
    protected String convertToHMS( int value ) {
	return( Convert.secToHHMMSS( value ));
    }
}

