package org.eso.ohs.persistence;

/**
 * An exception for authorisation failure exceptions from StorageManager
 * subclasses.  Typically, these are caught and result in a dialogue
 * to request the appropriate password for the relevant media,
 * eg ESO dbase.
 * 
 * <p>
 * $Id: ObjectAuthorisationException.java,v 1.9 2004/02/17 14:31:56 tcanavan Exp $
 * @author Bob Kemp
 */

import org.eso.ohs.core.utilities.ObjectIOException;

public class ObjectAuthorisationException extends ObjectIOException {

    /** media for which access requires more authorisation, eg password */
    private Media device_;

    /**
     * Construct an exception that does <em>not</em> contain another
     * exception.
     * 
     * @param msg   explanatory message giving a reason for the exception.
     */
    public ObjectAuthorisationException(Media device, String msg) {
	super(msg);

	device_ = device;
    }


    /**
     * Returns the media for which access requires more authorisation.
     * 
     * return Media object
     */
    public Media getMedia() {
	return device_;
    }

}

