package org.eso.ohs.persistence;


import org.eso.ohs.dfs.StorableObject;

    /**
     * A container class for several fields and used only with the
     * inMemory table (a.k.a. the Registry).
     * 
     * <b>NOTE:</b>  Some of these methods should perhaps be synchronized
     *               in future to protect against concurrent access but
     *		     at the moment there seems no need.
     */
    public class RegistryEntry {
	/** The object itself */
	private StorableObject sobj_;

	/** External media upon which the object is stored */
	private Media device_;

	/** StorageManager used to update the object on external media */
	private StorageManager stMgr_;

	/** Logical directory in which the object is stored on the media */
	private DirectoryNode dir_;

	/** No copy of the object exists on the medium? Important for DB */
	protected boolean isNew_;


	public RegistryEntry(StorableObject sobj, Media device,
				StorageManager stMgr, DirectoryNode dir,
				boolean isNew) {

	    if (sobj == null) {
		throw new NullPointerException("Bad Registry entry: no object");
	    }

	    sobj_   = sobj;
	    device_ = device;
	    stMgr_  = stMgr;
	    dir_    = dir;
	    isNew_  = isNew;
	}
  
        /**
         * Return a refencet to the Storable object that the 
         * RegistryEntry refers to
         * 
         * @return StorableObject
	 */

	public StorableObject getObject() {
	    return sobj_;
	}

        /**
         * The Media where the Registry Entry resides
         * @return Media Database,Local Storage
         */

	public Media getMedia() {
	    return device_;
	}

        /**
         * The storage manager that is used to store the 
         * BusinessObject to the database         
         */

	public StorageManager getStorageManager() {
	    return stMgr_;
	}

     
        /**
         * Set the directory associated with the Storable Object
         * @return dir Directory Node
         */


	public void setDirectory(DirectoryNode dir) {
	    dir_ = dir;
	}


        /**
         * Return the directort associated with the Storable Object
         * This does not apply to Dbase Objects
         * @return DirectortNode for business Objects
         */

	public DirectoryNode getDirectory() {
	    return dir_;
	}

        /**
         * Set the new value for the Registry Entry 
         * @param isNew 
         */

	public void setNew(boolean isNew) {
	    isNew_ = isNew;
	}

        /**
         * If the Registry Entry is already in the registry         
         * @return isNew
         */

	public boolean isNew() {
	    return isNew_;
	}

    }
