package org.eso.ohs.persistence.dbase.phase1;

import java.io.IOException;
import java.math.BigDecimal;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.Vector;

import org.apache.log4j.Logger;
import org.eso.ohs.core.utilities.ObjectIOException;
import org.eso.ohs.core.utilities.TextUtils;
import org.eso.ohs.dfs.ObsRun;
import org.eso.ohs.dfs.StorableObject;
import org.eso.ohs.persistence.Config;
import org.eso.ohs.persistence.ObjectNotFoundException;
import org.eso.ohs.persistence.SuspendedRunType;
import org.eso.ohs.persistence.StorageManager.UnpackedStorableObject;
import org.eso.ohs.persistence.dbase.DbaseIOHandler;

/**
 * I/O class for ObsRun stored in the database.
 * 
 * <p>
 * $Id: DbaseHandlerObsRun.java,v 1.8 2005/10/19 13:21:53 ddorigo Exp $<br>
 * 
 * @author Bob Kemp
 * 
 * 
 */

public class DbaseHandlerObsRun extends DbaseIOHandler {

    public static final String NIGHT_FLAG = "n";

    private static final String HOURS_FLAG = "h";

    public static final String BASIC_FLAG = "00000000000000000";

    public static final String FLAGS = "flags";

    public static final float INTFLOAT_CONV = 10;

    public static final float TIME_CONV = 10;

    public static final String VISITOR_MODE = "v";

    public static final String SERVICE_MODE = "s";

    public static final String[] RUN_DESC =
    { "A", "B", "C", "D", "E", "F", "G", "H", "I", "J", "K", "L", "M", "N",
            "O", "P", "Q", "R", "S", "T", "U", "V", "W", "X", "Y", "Z" };

    public static final Integer[] RUN_DESC_INT =
    { new Integer(0), new Integer(1), new Integer(2), new Integer(3),
            new Integer(4), new Integer(5), new Integer(6), new Integer(7),
            new Integer(8), new Integer(9), new Integer(10), new Integer(11),
            new Integer(12), new Integer(13), new Integer(14), new Integer(15),
            new Integer(16), new Integer(17), new Integer(18), new Integer(19),
            new Integer(20), new Integer(21), new Integer(22), new Integer(23),
            new Integer(24), new Integer(25), new Integer(26), new Integer(27) };

    final static String run_sql = "SELECT " + Phase1SelectStmt.runsTable.get()
            + ".id,  " + Phase1SelectStmt.runsTable.get() + ".run_desc, "
            + Phase1SelectStmt.runsTable.get() + ".note, "
            + Phase1SelectStmt.runsTable.get() + ".status, "
            + Phase1SelectStmt.runsTable.get() + ".run_code, "
            + Phase1SelectStmt.runsTable.get() + ".programme_id, "
            + "d_grade = " + Phase1SelectStmt.runsTable.get() + ".grade/"
            + INTFLOAT_CONV + ", " + Phase1SelectStmt.runsTable.get()
            + ".rank, " + Phase1SelectStmt.runsTable.get() + ".scheduled, "
            + Phase1SelectStmt.runsTable.get() + ".period, "
            + Phase1SelectStmt.runsTable.get() + ".supersede,"
            + Phase1SelectStmt.runsTable.get() + ".assigned_time,"
            + "d_assigned_time = " + Phase1SelectStmt.runsTable.get()
            + ".assigned_time/" + TIME_CONV + ", " + "d_sigma = "
            + Phase1SelectStmt.runsTable.get() + ".sigma/" + INTFLOAT_CONV
            + ", " + Phase1SelectStmt.runsTable.get() + ".obs_mode, "
            + Phase1SelectStmt.runsTable.get() + ".tel_code, "
            + Phase1SelectStmt.runsTable.get() + ".inst_code, "
            + Phase1SelectStmt.runsTable.get() + ".requested_time, "
            + Phase1SelectStmt.runsTable.get() + ".month, "
            + Phase1SelectStmt.runsTable.get() + ".moon, "
            + Phase1SelectStmt.runsTable.get() + ".seeing, "
            + Phase1SelectStmt.runsTable.get() + ".proprietary_time, "
            + Phase1SelectStmt.runsTable.get() + ".flags, "
            + Phase1SelectStmt.runsTable.get() + ".sky_transparency \n"
            + "FROM " + Phase1SelectStmt.runsTable.get() + " ";

    private static Logger stdlog_ = Logger.getLogger(DbaseHandlerObsRun.class);

    /**
     * Constructor for CS I/O class.
     */
    public DbaseHandlerObsRun() {
    }

    /**
     * Managed class
     * 
     * @return PanelObsRun class
     */

    public Class getManagedClass() {
        return ObsRun.class;
    }

    /**
     * Reads the object from the database, partially reassembling it. References
     * to other business objects are stored in other fields of the array
     * 
     * @param stmt
     *            JDBC Statement for contacting the dbase
     * @param id
     *            per-table id of the object to be retrieved.
     * @return array in the format required by reassembleReferences()
     * 
     * @exception SQLException
     *                an error occurred during the dbase operation
     * @exception ObjectNotFoundException
     *                the requested object was not found in the database
     * @exception IOException
     *                if an I/O error occurred while reading the object
     * 
     * @see org.eso.ohs.persistence.ObjectManager#disassembleReferences
     * @see org.eso.ohs.persistence.ObjectManager#reassembleReferences
     */
    public UnpackedStorableObject read(Statement stmt, long id)
            throws ObjectNotFoundException, IOException, SQLException {
        ResultSet rs = null;
        ObsRun panel_run = (ObsRun) newInstance();
        UnpackedStorableObject uso = new UnpackedStorableObject(panel_run);

        String sql = run_sql + "WHERE " + Phase1SelectStmt.runsTable.get()
                + ".id = " + id + "\n";

        stdlog_.debug(sql);
        rs = stmt.executeQuery(sql);

        if (!rs.next()) { // Only one with this id !!
            throw new ObjectNotFoundException("Object not found in database");
        }

        panel_run.setId(Config.getCfg().tableToUniqueId(id, ObsRun.class));
        setRunFields(panel_run, rs);

        rs.close();

        return uso;
    }

    /**
     * Read data
     * 
     * @param sql
     *            statement
     * @param whereClause
     *            where condition for retreiving the object
     * @exception SQLException
     *                an error occurred during the dbase operation
     * @exception ObjectNotFoundException
     *                the requested object was not found in the database
     * @exception ObjectIOException
     *                IO Eror occured while reading from the database
     */

    public UnpackedStorableObject[] read(Statement stmt, String whereClause,
            String otherTables)
            throws ObjectIOException, ObjectNotFoundException, SQLException {

        if (otherTables == null) {
            otherTables = "";
        }
        UnpackedStorableObject[] prog_array = null;
        ObsRun prog = null;
        String sql = buildSqlQuery(whereClause, otherTables);
        stdlog_.debug(sql);
        ResultSet rs = stmt.executeQuery(sql);

        try {
            Vector prog_vector = new Vector();

            while (rs.next()) {

                prog = (ObsRun) newInstance();
                // New element
                prog_vector.addElement(prog);

                // Set only when attaching the object as a new
                // element
                prog.setId(Config.getCfg().tableToUniqueId(rs.getInt("id"),
                        ObsRun.class));
                setRunFields(prog, rs);
            }
            rs.close();

            prog_array = new UnpackedStorableObject[prog_vector.size()];
            for (int i = 0; i < prog_vector.size(); i++) {
                prog = (ObsRun) prog_vector.elementAt(i);
                prog_array[i] = new UnpackedStorableObject(prog);
            }
        } catch (SQLException e) {
            e.printStackTrace();
        }

        return prog_array;
    }

    public static String buildSqlQuery(String whereClause, String otherTables) {
        String sql = run_sql + otherTables + " WHERE " + whereClause;
        return sql;
    }

    /**
     * Set the prog fields from the result set. Only for avoiding duplicated
     * code
     * 
     */
    protected void setRunFields(ObsRun run, ResultSet rs) throws SQLException {
        run.setTableId(rs.getInt("id"));
        run.setStatus(rs.getString("status"));
        run.setGrade(rs.getFloat("d_grade"));
        run.setScheduled(rs.getString("scheduled"));
        run.setPeriod(rs.getInt("period"));
        run.setShowCommentFlag(rs.getInt("supersede"));
        run.setNote(rs.getString("note"));
        run.setRank(rs.getInt("rank"));
        run.setAssignedTime(rs.getFloat("d_assigned_time"));
        run.setIntAssignedTime(rs.getInt("assigned_time"));
        run.setSigma(rs.getFloat("d_sigma"));
        run.setObsMode(rs.getString("obs_mode"));
        run.setTelescope(rs.getString("tel_code"));
        run.setInstrument(rs.getString("inst_code"));

        run.setReqTime(rs.getString("requested_time"));
        run.setMonth(rs.getString("month"));
        run.setMoon(rs.getString("moon"));
        run.setSeeing(rs.getString("seeing"));
        run.setProprietaryTime(rs.getInt("proprietary_time"));
        run.setSkyTransparency(rs.getString("sky_transparency"));
        run.setProgrammeId(rs.getLong("programme_id"));
        run.setRunDesc(rs.getString("run_desc"));
        run.setRunCode(rs.getString("run_code"));
        String flags = rs.getString(FLAGS);
        checkDelayedInstrument(run, flags);
    }

    /**
     * Writes the object to the database, for the first time.
     * 
     * @param stmt
     *            JDBC Statement for contacting the dbase
     * @param id
     *            per-table id of the object to be written.
     * @param obj
     *            an array: the first element is the BO to be stored.
     * @exception SQLException
     *                an error occurred during the dbase operation
     * 
     * @see org.eso.ohs.persistence.ObjectManager#disassembleReferences
     * @see org.eso.ohs.persistence.ObjectManager#reassembleReferences
     */
    public void write(Statement stmt, long id, UnpackedStorableObject obj)
            throws ObjectIOException {
        stdlog_.debug("the obj must be saved with id " + id);
        ObsRun obs = (ObsRun) obj.getObject();
        String sqlText = buildWriteStmt(obs);

        try {
            stmt.executeUpdate(sqlText);
        } catch (SQLException e) {
            stdlog_.error("INSERT error", e);
            throw new ObjectIOException("Error executing write: " + sqlText);
        }
        // throw new ObjectIOException("Operation not supported for this
        // object");
    }

    public static String buildWriteStmt(ObsRun obs) {
        long obsId = Config.getCfg().uniqueToTableId(obs.getId());
        Float f = new Float(obs.getGrade() * INTFLOAT_CONV);
        int grade = f.intValue();
        Float s = new Float(obs.getSigma() * INTFLOAT_CONV);
        int sigma = s.intValue();
        Float t = new Float(obs.getAssignedTime() * TIME_CONV);
        int assTime = t.intValue();
        String flags = buildFlags(obs);
        String sqlText = Phase1SelectStmt.beginTransaction + " \n "
                + "INSERT INTO " + Phase1SelectStmt.runsTable.get()
                + " (programme_id, run_desc, status, id, tel_code, \n"
                + "inst_code, requested_time, grade, assigned_time, \n"
                + "note, rank, scheduled, period, supersede, obs_mode, \n"
                + "month, moon, seeing, "
                + "rank_class, rated_flag, sigma, sky_transparency, \n"
                + "run_code, proprietary_time, flags) \n VALUES ("
                + obs.getProgrammeId() + ", \'" + obs.getRunDesc() + "\', \'"
                + obs.getStatus() + "\', " + obsId + ", \'"
                + obs.getTelescope() + "\', \n \'" + obs.getInstrument()
                + "\', \'" + obs.getReqTime() + "\', " + grade + ", " + assTime
                + ",\n \'" + obs.getNote() + "\', " + obs.getRank() + ", \'"
                + obs.getScheduled() + "\'," + obs.getPeriod() + ", 0 , \'"
                + obs.getObsMode() + "\', \'" + obs.getMonth() + "\', \'"
                + obs.getMoon() + "\', \'" + obs.getSeeing() + "\', \'A\', 0, "
                + sigma + ", \'" + obs.getSkyTransparency() + "\',\n \'"
                + obs.getRunCode() + "\', " + obs.getProprietaryTime() + ", \'"
                + flags + "\')";
        stdlog_.debug(sqlText);
        return sqlText;
    }

    /**
     * @param obs
     * @return String
     */
    private static String buildFlags(ObsRun obs) {
        StringBuffer flags = new StringBuffer(BASIC_FLAG);

        if (obs.isDelayedInstrument()) {
            flags.replace(SuspendedRunType.DELAYEDINSTRUMENT_FLAG_POS,
                    SuspendedRunType.DELAYEDINSTRUMENT_FLAG_POS + 1, "1");
        }
        return flags.toString();
    }

    /**
     * Updates an object that is already stored in the database.
     * 
     * @param stmt
     *            JDBC Statement for contacting the dbase
     * @param id
     *            per-table id of the object to be written.
     * @param obj
     *            an array: the first element is the BO to be stored.
     * @exception SQLException
     *                an error occurred during the dbase operation
     * 
     * @see org.eso.ohs.persistence.ObjectManager#disassembleReferences
     * @see org.eso.ohs.persistence.ObjectManager#reassembleReferences
     */
    public void update(Statement stmt, StorableObject obj) throws SQLException {
        ObsRun panel_run = (ObsRun) obj;

        ObsRun or = (ObsRun) obj;
        String flags = buildFlags(or);

        StringBuffer sb = new StringBuffer();

        sb.append(" tel_code = '");
        sb.append(TextUtils.quoteSqlString(panel_run.getTelescope()));
        sb.append("', inst_code = '");
        sb.append(TextUtils.quoteSqlString(panel_run.getInstrument()));
        sb.append("', requested_time = '");
        sb.append(TextUtils.quoteSqlString(panel_run.getReqTime()));
        sb.append("', grade = ");
        sb.append(panel_run.getGrade() * INTFLOAT_CONV);
        sb.append(", assigned_time = ");
        sb.append(panel_run.getAssignedTime() * TIME_CONV);
        sb.append(", rank = ");
        sb.append(panel_run.getRank());
        sb.append(", proprietary_time = ");
        sb.append(panel_run.getProprietaryTime());
        sb.append(", scheduled = '");
        sb.append(TextUtils.quoteSqlString(panel_run.getScheduled()));
        sb.append("', period = ");
        sb.append(panel_run.getPeriod());
        sb.append(", supersede = ");
        sb.append(panel_run.getShowCommentFlag());
        sb.append(", status = '");
        sb.append(TextUtils.quoteSqlString(panel_run.getStatus()));
        sb.append("', obs_mode = '");
        sb.append(TextUtils.quoteSqlString(panel_run.getObsMode()));
        sb.append("', run_code = '");
        sb.append(TextUtils.quoteSqlString(panel_run.getRunCode()));
        sb.append("', sigma = ");
        sb.append(panel_run.getSigma() * INTFLOAT_CONV);
        sb.append(", flags = '");
        sb.append(TextUtils.quoteSqlString(flags));
        sb.append("', note = '");
        sb.append(TextUtils.quoteSqlString(panel_run.getNote()));
        sb.append("'");
        sb.append("\n");

        String sqlText = "UPDATE " + Phase1SelectStmt.runsTable.get() + ""
                + " SET " + sb + " WHERE id = "
                + Config.getCfg().uniqueToTableId(panel_run.getId());

        stdlog_.debug(sqlText);
        stmt.executeUpdate(sqlText);
    }

    /**
     * Updates an object that is already stored in the database.
     * 
     * @param stmt
     *            JDBC Statement for contacting the dbase
     * @param id
     *            per-table id of the object to be written.
     * @param obj
     *            an array: the first element is the BO to be stored.
     * @exception SQLException
     *                an error occurred during the dbase operation
     * 
     * @see org.eso.ohs.persistence.ObjectManager#disassembleReferences
     * @see org.eso.ohs.persistence.ObjectManager#reassembleReferences
     */
    public void update(Statement stmt, long id, UnpackedStorableObject objSpec)
            throws SQLException {
        stdlog_.debug("update object");
        StorableObject obj = objSpec.getObject();
        update(stmt, obj);
        stdlog_.debug("update object done");
    }

    public static long buildRunId(long progId, String runDescription)
            throws ObjectIOException {
        int runIncrInt = -1;
        long runId = 0;
        for (int j = 0; j < DbaseHandlerObsRun.RUN_DESC.length; j++) {
            if (runDescription.equals(DbaseHandlerObsRun.RUN_DESC[j])) {
                runIncrInt = DbaseHandlerObsRun.RUN_DESC_INT[j].intValue();
                break;
            }
        }
        if (runIncrInt >= 0) {
            runId = runIncrInt + progId * 10;
        } else {
            throw new ObjectIOException(
                    "run description invalid value must be [A-Z]: \""
                            + runDescription + "\"");
        }
        return runId;
    }

    /**
     * Deletes an object from the database.
     * 
     * @param stmt
     *            JDBC Statement for contacting the dbase
     * @param id
     *            per-table id of the object to be written.
     * @exception SQLException
     *                an error occurred during the dbase operation
     */
    public void delete(Statement stmt, long id) throws ObjectIOException {
        throw new ObjectIOException("Operation not supported for this object");
    }

    /**
     * @param prog
     * @param flags
     */
    private void checkDelayedInstrument(ObsRun run, String flags) {
        if (flags != null
                && flags.length() > SuspendedRunType.DELAYEDINSTRUMENT_FLAG_POS
                && flags.charAt(SuspendedRunType.DELAYEDINSTRUMENT_FLAG_POS) == SuspendedRunType.DELAYEDINSTRUMENT_FLAG) {
            run.setDelayedInstrument(true);
        } else {
            run.setDelayedInstrument(false);
        }
    }

    /**
     * @param time
     * @param mode
     * @return int
     * @throws ObjectIOException
     */
    public static BigDecimal convertReqTimeToBig(String time, String mode)
            throws ObjectIOException {
        BigDecimal timeBig = new BigDecimal("0");
        if (DbaseHandlerObsRun.SERVICE_MODE.equals(mode)) {
            int hPos = time.indexOf(HOURS_FLAG);
            if (hPos > 0) {
                String timeIntString = time.substring(0, hPos);
                timeBig = new BigDecimal(timeIntString);
            } else {
                throw new ObjectIOException(
                        " missing '"
                                + HOURS_FLAG
                                + "' in time parameter: in service mode time is expressed in hours.");
            }
        } else if (DbaseHandlerObsRun.VISITOR_MODE.equals(mode)) {
            int nPos = time.indexOf(NIGHT_FLAG);
            if (nPos > 0) {
                String timeIntString = time.substring(0, time
                        .indexOf(NIGHT_FLAG));
                timeBig = new BigDecimal(timeIntString);
            } else {
                throw new ObjectIOException(
                        " missing '"
                                + NIGHT_FLAG
                                + "' in time parameter: in visitor mode time is expressed in nights.");
            }
        } else {
            throw new ObjectIOException(
                    "Time cannot be validated: Observing Mode is wrong");
        }
        return timeBig;
    }
}