package org.eso.ohs.phase2.actions;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.util.Vector;

import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JOptionPane;

import org.eso.ohs.core.gui.baseaction.ActionSuperclass;
import org.eso.ohs.core.gui.baseaction.IncrementalAction;
import org.eso.ohs.core.gui.baseaction.IncrementalActionAutomaton;
import org.eso.ohs.core.gui.widgets.ErrorMessages;
import org.eso.ohs.core.gui.widgets.TextDisplayWidget;
import org.eso.ohs.core.utilities.Convert;
import org.eso.ohs.core.utilities.ObjUtils;
import org.eso.ohs.core.utilities.ObjectIOException;
import org.eso.ohs.core.utilities.TextUtils;
import org.eso.ohs.dfs.CalibrationBlock;
import org.eso.ohs.evm.EVMException;
import org.eso.ohs.evm.EvmExecutorImpl;
import org.eso.ohs.instruments.Instrument;
import org.eso.ohs.instruments.InstrumentList;
import org.eso.ohs.persistence.Config;
import org.eso.ohs.persistence.Media;
import org.eso.ohs.persistence.ObjectContainer;
import org.eso.ohs.persistence.ObjectManager;
import org.eso.ohs.persistence.ObjectNotFoundException;

/**
 * Action class to display the detailed contents of a BusinessObject.
 *
 * <p>
 * $Id: ExpTimeReportAction.java,v 1.16 2005/04/14 09:41:30 tcanavan Exp $
 * <br>
 * @author Bob Kemp
 * 
 * <br>ChangeLog: cmavros 11/Sept/01 Moved ConvertToHoursMinsSecs(int) to utilities class
 * <br>ChangeLog: cmavros 10/10/01 Cleaned up printout 
 * 
 */

public class ExpTimeReportAction extends ActionSuperclass {

    private static final int ID_PADDING = 9;
    private static final int EXTIME_PADDING = 15;
    private static final int HDING_PADDING = 59;
    private static final int OBSRUN_ID_PADDING = 14;
    //private static final int OB_NAME_PADDING = 26;   

    private static TextDisplayWidget fileMenu_ =
        new TextDisplayWidget("Execution Time Report");

    protected ObjectContainer folderView_;

    protected boolean folder_;

    /**
     * Constructs a ShowDetailsAction action.
     */
    public ExpTimeReportAction(
        ObjectContainer folderView,
        boolean folder,
        String name) {
        super((JComponent) folderView, name);

        folderView_ = folderView;
        folder_ = folder;
        //fileMenu_.setVisible(false);
    }

    /**
     * Event handler for the ShowDetailsAction action.
     * 
     * @param ev       user-initiated ActionEvent to quit.
     */
    public void actionPerformedImpl(ActionEvent evt) {
        Long[] idLongs = null;
        if (!folder_) {
            idLongs = folderView_.getAllSelected();
        } else {
            idLongs = folderView_.getAll();
        }

        idLongs = (Long[]) ObjUtils.uniqueOccurrences(idLongs);
        //Add error message if nothing selected 09/11/01 CMM
        if (idLongs == null || idLongs.length <= 0) {
            JOptionPane.showMessageDialog(
                folderView_.getTopLevelAncestor(),
                new Object[] {
                    "There are no selected business objects",
                    " ",
                    "You must first select business objects to report on." },
                "Nothing selected",
                JOptionPane.ERROR_MESSAGE);
            return;
        }
        new IncrementalActionAutomaton(
            new IncrementalReport(idLongs),
            (JFrame) folderView_.getTopLevelAncestor(),
            "Reports...");
    }

    /**
     * Utility class to export a sequence of Business Objects, one at a time.
     * The user is also given the opportunity to stop the process using
     * a dialog box.
     */
    public class IncrementalReport implements IncrementalAction {

        /** array of ids of objects to be duplicated */
        private Long[] idLongs_;

        /** Type of business object to be duplicated */
        private Class objType_;

        /** Media containing the objects to be duplicated */
        private Media device_;

        private int totExpTime_;

        private int totExecTime_;

        private int numProcessed_;

        private int failedVerify_;

        private Vector failedOBs_;
        
        private Vector failedExecScriptOBs_;

        StringBuffer buf = new StringBuffer();

        /**
         * Constructor.
         * 
         * @param device    media containing the objects to be duplicated
         * @param objType   type of object to be duplicated
         * @param idLongs   array of ids of objects to be duplicated
         */
        public IncrementalReport(Long[] idLongs) {
            idLongs_ = idLongs;
        }

        public int getNumStepsRemaining(int counter) {
            return ObjUtils.howManyNotNull(idLongs_, counter);
        }

        public void startOfAction() {
            failedOBs_ = new Vector();
            failedExecScriptOBs_ = new Vector();             
            fileMenu_.clearTextArea();
            fileMenu_.setVisible(false);
            totExpTime_ = 0;
            totExecTime_ = 0;
            numProcessed_ = 0;
            failedVerify_ = 0;
            buf.append(
                "OB Execution Time Report "
                    + "("
                    + TextUtils.currentISODateTime()
                    + "(UT))\n");
            buf.append("==================================================\n");
            buf.append("\n");
            buf.append(
                "  Programme Id  "
                    + "      Id"
                    + " | OB Name                           "
                    + "Exposure Time"
                    + "       Execution Time\n");
            buf.append(
                " -------------- "
                    + "  --------------------------------           "
                    + "-------------"
                    + "       --------------\n");
        }

        public void endOfAction() {            
            if (numProcessed_ > 0) {
                String expTotal = Convert.secToHHMMSS(totExpTime_);
                String execTotal = Convert.secToHHMMSS(totExecTime_);
                buf.append("\n");
                for (int i = 0; i < HDING_PADDING; i++)
                    buf.append(" ");
                buf.append("  -------------       --------------\n");
                buf.append("  Totals");
                for (int i = 0; i < (HDING_PADDING - 11); i++)
                    buf.append(" ");
                buf.append("  ");
                for (int i = 0; i < (EXTIME_PADDING - expTotal.length()); i++)
                    buf.append(" ");
                buf.append(expTotal + "     ");
                for (int i = 0; i < (EXTIME_PADDING - execTotal.length()); i++)
                    buf.append(" ");
                buf.append(execTotal + "\n");
            }
            if (failedVerify_ > 0) {
                buf.append(
                    "\n\n==================================================\n");
                buf.append("\n" + failedVerify_ + " Failed Verification\n");
                if (failedOBs_.size() > 0) {
                    buf.append("\n");
                    for (int i = 0; i < failedOBs_.size(); i++) {
                        buf.append(failedOBs_.elementAt(i));
                    }
                }
            }            
            int failedExecScr  = failedExecScriptOBs_.size();
            if (failedExecScr > 0) {
                buf.append(
                    "\n\n==================================================\n");
                buf.append("\n" + failedExecScr +  " OBs required Execution Time Scripts that were missing or contained errors\n");
                if (failedExecScr > 0) {
                    buf.append("\n");
                    for (int i = 0; i < failedExecScr; i++) {
                        buf.append(failedExecScriptOBs_.elementAt(i));
                    }
                }
            }

            if (numProcessed_ <= 0)
                buf.append("\nUnable to report on any of the selected ob's\n");
            buf.append(
                "\n\n==================================================\n");
            fileMenu_.setTextArea(buf.toString());
            fileMenu_.setSize(new Dimension(725, 400));
            fileMenu_.setVisible(true);
        }

        /**
         * Method to duplicate the n'th element of obs, where n = counter.
         * 
         * @param counter   counter to show which element of obs to check in.
         * @return	    true if the duplicating should continue
         *		    and false otherwise.
         */
        public void performStep(int counter) {
            if (idLongs_[counter] != null) {
                // Main body of the "loop" + fall thru
                doExpTime(idLongs_[counter].longValue());
            }
        }

        /**
         * Utility method produce a string that calculates the exposure/execution time
         * 
         * @param id	    id of business object to duplicate
         */
        public void doExpTime(long id) {
            String failedOBsName = "";
            boolean noExecTimeAvailable = false;
            long obid = 0;
            try {
                device_ = Config.getCfg().getMediaFromId(id);
                objType_ = Config.getCfg().getClassFromId(id);
                ObjectManager objMgr = ObjectManager.getObjectManager();
                CalibrationBlock obj =
                    (CalibrationBlock) objMgr.getBusObj(device_, id, objType_);

                int i = 0;
                int padlen = 0;
                String obsName = obj.getObsRun().getName();
                String obsId = obsName.substring(0, obsName.indexOf('/'));
                padlen = OBSRUN_ID_PADDING - obsId.length();
                long dbaseId = obj.getDbaseId();
                if (EvmExecutorImpl.evmConfigured()) {
                    EvmExecutorImpl evm = EvmExecutorImpl.getInstance();
                    //Check that the evm is initialized
                    Instrument inst =
                        InstrumentList.getInstance().getInstrument(
                            obj.getOd().getInstrument(),obj.fetchIPVersionFromOd());
                           
                    
                    if (evm.isInitialized()) {
                        if (!inst.hasExecTimeCalc()) {
                               noExecTimeAvailable = true;               
                        }
                    }
                } else {
                    noExecTimeAvailable = true;
                }
                if(noExecTimeAvailable) {
                    // add to the failedExecScript set and return                  
                    StringBuffer buf1 = new StringBuffer();
                    buf1.append(" ");
                    buf1.append(obsId);
                    buf1.append("  ");
                    padlen = getPadLen(dbaseId);
                    obid = getOBId(obid, padlen, dbaseId);
                    buf1.append(new Long(obid).toString());
                    buf1.append("  ");
                    buf1.append(obj.getName());
                    buf1.append("\n");
                    failedExecScriptOBs_.addElement(buf1);
                    return;
                }
                                
                if (obj.verify().length() > 0) {
                    failedOBs_.addElement(" ");
                    failedOBs_.addElement(obsId);
                    failedOBs_.addElement("  ");
                    padlen = getPadLen(dbaseId);
                    obid = getOBId(obid, padlen, dbaseId);
                    failedOBs_.addElement(new Long(obid).toString());
                    failedOBs_.addElement("  ");
                    failedOBs_.addElement(obj.getName());
                    failedOBs_.addElement("\n");
                    failedVerify_++;
                    return;
                }
                else {
                    try {
                        EvmExecutorImpl evm = EvmExecutorImpl.getInstance();
                        evm.calculateETCS(obj);
                        obj.setExecTimeUpToDate(true);
                    }
                    catch(EVMException e) {
                        StringBuffer buf1 = new StringBuffer();
                        buf1.append(" ");
                        buf1.append(obsId);
                        buf1.append("  ");
                        padlen = getPadLen(dbaseId);
                        obid = getOBId(obid, padlen, dbaseId);
                        buf1.append(new Long(obid).toString());
                        buf1.append("  ");
                        buf1.append(obj.getName());
                        buf1.append("\n");
                        failedExecScriptOBs_.addElement(buf1);
                        return;
                        
                    }    

                }
                for (i = 0; i < padlen; i++) {
                    buf.append(" ");
                }                
                buf.append(obsId);
                buf.append("  ");
                padlen = 0;
                if (dbaseId != 0) {
                    padlen =
                        (ID_PADDING - new Long(dbaseId).toString().length());
                    if (!device_.equals(Media.DBASE)) {
                        padlen += 2;
                    }
                } else
                    padlen = ID_PADDING;
                for (i = 0; i < padlen; i++)
                    buf.append(" ");
                if (dbaseId != 0) {
                    obid = 0;
                    if (device_.equals(Media.DBASE)) {
                        obid = dbaseId;
                    } else
                        obid = Config.getCfg().uniqueToTableId(dbaseId);
                    buf.append(new Long(obid).toString());
                }
                buf.append(" ");
                String name = null;
                padlen = 0;
                int nameLen = Config.getCfg().getMaxNameLength();
                if (obj.getName().length() > nameLen)
                    name = obj.getName().substring(0, nameLen);
                else
                    name = obj.getName();
                buf.append(name);
                if (obj.getName().length() < nameLen)
                    padlen = nameLen - name.length();
                for (i = 0; i < padlen; i++)
                    buf.append(" ");

                /** Get Execution Time */
                int execTime = (int) obj.getExecutionTime();

                /** Display execution and exposure time */

                String xTime =
                    Convert.secToHHMMSS(
                        (int) ((CalibrationBlock) obj).getExposureTime());
                for (i = 0; i < (EXTIME_PADDING - xTime.length()); i++)
                    buf.append(" ");
                buf.append(xTime);
                buf.append("     ");
                //Check that execution time is valid
                if (execTime < 0 || execTime > Convert.EXECTIME_MAX)
                    xTime = Convert.EXECTIME_ERR_STRING;
                else
                    xTime = Convert.secToHHMMSS((int) execTime);
                for (i = 0; i < (EXTIME_PADDING - xTime.length()); i++)
                    buf.append(" ");
                buf.append(xTime);

                buf.append("\n");

                /** Total execution and exposure time */

                totExpTime_ = totExpTime_ + (int) obj.getExposureTime();
                //Check that execution time is valid
                if (execTime < 0 || execTime > Convert.EXECTIME_MAX)
                    totExecTime_ = totExecTime_ + Convert.EXECTIME_ERR;
                else
                    totExecTime_ = totExecTime_ + execTime;

                numProcessed_++;
            } catch (ObjectNotFoundException e) {
                ErrorMessages.announceNoObject(
                    (JFrame) folderView_.getTopLevelAncestor(),
                    e);
            } catch (ObjectIOException e) {
                ErrorMessages.announceIOError(
                    (JFrame) folderView_.getTopLevelAncestor(),
                    e);
            } catch (RuntimeException e) {
                e.printStackTrace();
                throw e;
            } catch (Exception e) {
                e.printStackTrace();
            }
        }

        public int getPadLen(long dbaseId) {
            int padlen;
            if (dbaseId != 0)
                padlen =
                    (ID_PADDING
                        - new Long(dbaseId).toString().length());
            else
                padlen = ID_PADDING;
            return padlen;
        }

        public long getOBId(long obid, int padlen, long dbaseId) {
            int i;
            for (i = 0; i < padlen; i++)
                failedOBs_.addElement(" ");
            if (dbaseId != 0) {
                obid = 0;
                if (device_.equals(Media.DBASE)) {
                    obid = dbaseId;
                } else
                    obid = Config.getCfg().uniqueToTableId(dbaseId);
            }
            return obid;
        }
    }

}
