package org.eso.ohs.phase2.actions;
import java.awt.Dimension;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.event.ActionEvent;
import java.awt.image.BufferedImage;
import java.awt.print.PageFormat;
import java.awt.print.Printable;
import java.awt.print.PrinterException;
import java.awt.print.PrinterJob;
import java.util.Hashtable;

import javax.swing.ImageIcon;
import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;

import org.eso.ohs.core.gui.baseaction.ActionSuperclass;
import org.eso.ohs.core.gui.baseaction.IncrementalAction;
import org.eso.ohs.core.gui.baseaction.IncrementalActionAutomaton;
import org.eso.ohs.core.gui.utilities.ImageUtils;
import org.eso.ohs.core.gui.widgets.ErrorMessages;
import org.eso.ohs.core.utilities.ObjUtils;
import org.eso.ohs.core.utilities.ObjectIOException;
import org.eso.ohs.dfs.FindingChart;
import org.eso.ohs.dfs.ObservationBlock;
import org.eso.ohs.persistence.Config;
import org.eso.ohs.persistence.Media;
import org.eso.ohs.persistence.OHSPersistence;
import org.eso.ohs.persistence.ObjectContainer;
import org.eso.ohs.persistence.ObjectNotFoundException;
import org.eso.ohs.persistence.ObjectSelector;

/**
 * Action call to print finding charts.
 * This takes the original finding chart and 
 * prints it scaled.
 * <p>
 * $Id: PrintFindingCharts.java,v 1.14 2005/02/11 10:31:49 tcanavan Exp $
 * @author Tim Canavan
 */

public class PrintFindingCharts extends ActionSuperclass {
    /** medium on which the business object is to be created */
    private Media device_;

    /** view of the folder in which the OB is to be created */
    private ObjectContainer folderView_;
    
    
    private ObjectSelector objectSelector_;

    private boolean version2_;

    private int returnValue_;

    private boolean allOBs_ = false;
    
    private JFrame frm_;

    private PrinterJob prnJob;
    
    private JLabel title_;


    /**
     * Construct an action for finding charts
     * @param device
     * @param folderView
     * @param name
     * @param allOBs
     */


    public PrintFindingCharts(
        Media device,
        ObjectContainer folderView,
        String name,
        boolean allOBs) {
        super((JComponent) folderView, name);
        device_ = device;
        folderView_ = folderView;
        allOBs_ = allOBs;
        //frm_ = (JFrame)folderView_.getTopLevelAncestor();
    }
    
	public PrintFindingCharts(
		Media device,
		ObjectSelector objSelector,
		String name
		) {
		super((JFrame)objSelector, name);
		device_ = device;
		objectSelector_ = objSelector;
		frm_ = (JFrame)objSelector;
	}


    /**
     * Event handler for printing of Finding Charts.
     * 
     * @param ev user-initiated ActionEvent to create an OD.
     */
    public void actionPerformedImpl(ActionEvent ev) {
		Long[] idLongs;
    	if(objectSelector_!=null) {
    		idLongs = new Long[1];
    		idLongs[0] = new Long(objectSelector_.getDisplayedObject().getId());     		
    	} else {
			if (allOBs_)
				idLongs = folderView_.getAll();
			else
				idLongs = folderView_.getAllSelected();    		
    	}
        idLongs = (Long[]) ObjUtils.uniqueOccurrences(idLongs);
        if (idLongs.length == 0) {
            JOptionPane.showMessageDialog(
                folderView_.getTopLevelAncestor(),
                new Object[] {
                    "There are no OB's Selected",
                    "Please select an OB ",
                    },
                "Nothing selected",
                JOptionPane.ERROR_MESSAGE);
            return;
        }
        if(folderView_ != null)
            frm_ = (JFrame)folderView_.getTopLevelAncestor();

        new IncrementalActionAutomaton(
            new IncrementalPrint(device_, idLongs),
            frm_,
            "Printing...");
    }

    /**
     * Utility class to export a sequence of Business Objects, one at a time.
     * The user is also given the opportunity to stop the process using
     * a dialog box.
     */
    public class IncrementalPrint implements IncrementalAction {

        /** cached reference to Persistence */
        private OHSPersistence objMgr =
            (OHSPersistence) OHSPersistence.getObjectManager();

        private Long[] idLongs_;

        private Media device_;

        private Hashtable alreadyPrinted = new Hashtable();

        private PrinterJob prnJob;
        
        

        /**
         * Constructor.
         * 
         * @param device    media containing the objects to be duplicated
         * @param idLongs   array of ids of objects to be duplicated
         */
        public IncrementalPrint(Media device, Long[] idLongs) {
            device_ = device;
            idLongs_ = idLongs;
            prnJob = PrinterJob.getPrinterJob();
            if (!prnJob.printDialog())
                return;

        }

        public int getNumStepsRemaining(int counter) {
            return ObjUtils.howManyNotNull(idLongs_, counter);
        }

        public void startOfAction() {
            returnValue_ = -1;
        }

        public void endOfAction() {
            if (alreadyPrinted.size() <= 0) {
                JOptionPane.showMessageDialog(
                    folderView_.getTopLevelAncestor(),
                    new Object[] {
                        "No Finding Charts Found",
                        "Please select OB's with finding charts",
                        },
                    "No Finding Charts",
                    JOptionPane.ERROR_MESSAGE);
                return;
            }

        }

        /**
         * Method to print finding charts for OB's
         * 
         * @param counter   counter to show which element of obs to check in.
         * @return      true if the duplicating should continue
         *          and false otherwise.
         */
        public void performStep(int counter) {
            if (idLongs_[counter] != null) {
                // Main body of the "loop" + fall thru
                doPrint(idLongs_[counter].longValue());
            }
        }

        /**
         * Utility method to duplicate a given business object
         * 
         * @param id        id of business object to duplicate
         */
        public  void doPrint(long id) {
            try {
                JPEGPanel panel = new JPEGPanel();
                Class c = Config.getCfg().getClassFromId(id);
                if (c.equals(ObservationBlock.class)) {
                    ObservationBlock ob = (ObservationBlock)objMgr.getBusObj(Media.DBASE,id,ObservationBlock.class);
                    title_ = new JLabel(ob.getObsRun().getName());
                    FindingChart[] fcCharts =
                        objMgr.populateFindingCharts(Media.DBASE, id);
                    for (int i = 0; i < fcCharts.length; i++) {
                        if (!alreadyPrinted
                            .containsKey(new Long(fcCharts[i].getCheckSum()))) {
                            ImageIcon imIcon =
                                new ImageIcon(fcCharts[i].getFcData());
                            BufferedImage img =
                                ImageUtils.convertToBufferedImage(
                                    imIcon.getImage(),
                                    imIcon.getIconWidth(),
                                    imIcon.getIconHeight(),
                                    1);
                            panel.setBufferedImage(img);
                            prnJob.setPrintable(panel);
                            alreadyPrinted.put(
                                new Long(fcCharts[i].getCheckSum()),
                                new Long(fcCharts[i].getCheckSum()));
                            prnJob.print();
                        }
                    }
                }
            } catch (ObjectIOException e) {
                ErrorMessages.announceIOError(
                    folderView_.getTopLevelAncestor(),
                    e);
            } catch (RuntimeException e) {
                e.printStackTrace();
                throw e;
            } catch (PrinterException e) {
                String [] text = new String[1];
                text[0] = "Printer Error";
                ErrorMessages.announceError(folderView_.getTopLevelAncestor(),text,e);
            } catch (ObjectNotFoundException e) {
                ErrorMessages.announceNoObject(folderView_.getTopLevelAncestor(),e);
            } 
        }
    }


    public Dimension getDim(
        int pageWidth,
        int pageHeight,
        int imageWidth,
        int imageHeight) {
        double magX = (double) imageWidth / (double) pageWidth;
        double magY = (double) imageHeight / (double) pageHeight;
        int dwidth = 0, dheight = 0;
        double mag = Math.max(magX, magY);
        Double db = new Double(imageWidth / mag);
        dwidth = db.intValue();
        db = new Double(imageHeight / mag);
        dheight = db.intValue();
        return new Dimension(dwidth, dheight);
    }
    
    
    // move later.

    public class JPEGPanel extends JPanel implements Printable {
        private BufferedImage bufferedImage;

        public int print(Graphics pg, PageFormat pageFormat, int pageIndex) {
            if (pageIndex >= 1)
                return NO_SUCH_PAGE;
            //Graphics2D 2d = (Gaphics2D)
            
            pg.translate(
                (int) pageFormat.getImageableX(),
                (int) pageFormat.getImageableY());
            int wPage = (int) pageFormat.getImageableWidth();
            int hPage = (int) pageFormat.getImageableHeight();
            if (bufferedImage == null)
                return NO_SUCH_PAGE;

            int w = bufferedImage.getWidth();
            int h = bufferedImage.getHeight();

            Dimension d = null;
            if (h < hPage && w < wPage) {
                d = new Dimension(w,h);
            } else {
                d = getDim(wPage, hPage, w, h);                    
            }
            FontMetrics fm = pg.getFontMetrics();    
            int y=fm.getAscent();
            System.out.println("Printing text " + title_.getText());
            //pg.
            pg.drawString(title_.getText(),0,y);                            
            pg.drawImage(
                bufferedImage,
                0,
                y+5,
                (int) d.getWidth(),
                (int) d.getHeight(),
                this);
            System.gc();
            return PAGE_EXISTS;
        }

        /**
         * Returns the bufferedImage.
         * @return BufferedImage
         */
        public BufferedImage getBufferedImage() {
            return bufferedImage;
        }

        /**
         * Sets the bufferedImage.
         * @param bufferedImage The bufferedImage to set
         */
        public void setBufferedImage(BufferedImage bufferedImage) {
            this.bufferedImage = bufferedImage;
        }

    }

}
