package org.eso.ohs.phase2.common;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.HeadlessException;
import java.awt.Image;
import java.awt.MediaTracker;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.InputEvent;
import java.awt.event.KeyEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.File;
import java.io.IOException;

import javax.swing.AbstractAction;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JToolBar;
import javax.swing.KeyStroke;

import org.eso.ohs.core.gui.widgets.ErrorMessages;
import org.eso.ohs.core.utilities.ObjectIOException;
import org.eso.ohs.dfs.FindingChart;
import org.eso.ohs.dfs.ObservationBlock;
import org.eso.ohs.dfs.StorableObject;
import org.eso.ohs.persistence.Media;
import org.eso.ohs.persistence.OHSPersistence;
import org.eso.ohs.persistence.ObjectManager;
import org.eso.ohs.persistence.ObjectNotFoundException;
import org.eso.ohs.persistence.ObjectSelectionEvent;
import org.eso.ohs.persistence.ObjectSelectionListener;

/**
 * @author tcanavan
 *
 * To change this generated comment e dit the template variable "typecomment":
 * Window>Preferences>Java>Templates. To enable and disable the creation of type
 * comments go to Window>Preferences>Java>Code Generation.
 */
public class SimpleJpegViewer
    extends JFrame
    implements ObjectSelectionListener, PropertyChangeListener {

    private JLabel statusStr = new JLabel();

    private static SimpleJpegViewer jpegViewer;

    private int currentRow;

    private JButton cancelBtn = new JButton("Cancel");

    private JButton nextBtn = new JButton("Next");

    private JButton prevBtn = new JButton("Previous");

    private JLabel statusLabel = new JLabel();

    private Media device_ = null;

    /** Object that is to be displayed in the ObjectView */
    private ObservationBlock ob_;

    private JButton comp = new JButton();

    private int maxWidth;

    private int maxHeight;

    private int currentIndex;

    public static SimpleJpegViewer getInstance() {
        if (jpegViewer == null) {
            jpegViewer = new SimpleJpegViewer();
        }
        return jpegViewer;
    }

    /**
     * Constructor for JPEGViewer.
     * @throws HeadlessException
     */
    private SimpleJpegViewer() throws HeadlessException {
        super();
        getContentPane().setLayout(new BorderLayout());
        Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
        maxWidth = (screenSize.width / 2);
        maxHeight = ((screenSize.height / 4) * 3);
        JToolBar toolBar = new JToolBar();
        JPanel south = new JPanel(true);
        getContentPane().add(comp);
        south.setLayout(new BoxLayout(south, BoxLayout.X_AXIS));
        south.add(cancelBtn);
        cancelBtn.addActionListener(new CancelActionListener());
        nextBtn.addActionListener(new NextActionListener());
        prevBtn.addActionListener(new PrevActionListener());
        south.add(nextBtn);
        south.add(prevBtn);
        south.add(Box.createGlue());
        south.add(statusLabel);
        getContentPane().add(south, BorderLayout.SOUTH);

        addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent ev) {
                setVisible(false);
            }
        });
        
        JMenuBar mb = new JMenuBar();
        String menuName = "File";
        JMenu menu = new JMenu(menuName);
        JMenuItem close = new JMenuItem("Close");
        close.setAccelerator(KeyStroke.getKeyStroke(
            KeyEvent.VK_W, InputEvent.CTRL_MASK, false));
        close.addActionListener(new ActionListener() {       
             public void actionPerformed(ActionEvent e) {
                  setVisible(false);
             }
        });
        menu.add(close);    
        mb.add(menu);
        setJMenuBar(mb);
    }

    public Dimension getScaleX(int imageWidth, int imageHeight) {
        double magX = (double) imageWidth / (double) maxWidth;
        double magY = (double) imageHeight / (double) maxHeight;
        double mag = Math.max(magX, magY);
        int width = (int) (imageWidth / mag);
        int height = (int) (imageHeight / mag);
        Dimension d = new Dimension(width, height);
        return d;
    }


    public void init(ObservationBlock ob)
        throws IOException, ObjectNotFoundException, ObjectIOException {
        setDisplayedObject(ob);
        currentIndex = 0;
        FindingChart[] fcCharts = ob.getFindingCharts();
        if(fcCharts.length == 0) {
            makeTitle();            
            clear();
            return;
        }        
        for (int i = 0; i < fcCharts.length; i++) {
            fcCharts[i].setObsRun(ob.getObsRun());
        }
        OHSPersistence persistence =
            (OHSPersistence) OHSPersistence.getObjectManager();
        displayCurrentFindingChart(fcCharts[0]);
        setPrevNextButton();
        makeTitle();
        setVisible(true);
        pack();

    }

    public void clear() {
        comp.setIcon(null);
        prevBtn.setEnabled(false);
        nextBtn.setEnabled(false);
        statusLabel.setText("");
    }

    public void displayCurrentFindingChart(FindingChart fcChart)
        throws IllegalArgumentException, ObjectNotFoundException {
        OHSPersistence persistence =
            (OHSPersistence) OHSPersistence.getObjectManager();
        String folderPath = persistence.getRootFolderPath(ob_.getObsRun());
        Image img =
            Toolkit.getDefaultToolkit().getImage(
                folderPath
                    + File.separator
                    + fcChart.getCheckSum()
                    + "."
                    + "jpg");
        MediaTracker mediaTracker = new MediaTracker(this);
        mediaTracker.addImage(img, 0);
        try {
            mediaTracker.waitForID(0);
        } catch (InterruptedException ie) {
            // TODO put Error Message in.
            //  System.exit(1);
        }
        Dimension bufDim = getScaleX(img.getWidth(null), img.getHeight(null));
        img =
            img.getScaledInstance(
                bufDim.width,
                bufDim.height,
                Image.SCALE_DEFAULT);
        ImageIcon imgIcon = new ImageIcon(img);
        comp.setIcon(imgIcon);
        makeTitle();
        setStatusString();
    }

    public void makeTitle() {
        FindingChart [] fcs = ob_.getFindingCharts();
        if(fcs.length == 0) {
           setTitle(ob_.getName());
	    } else {
           setTitle(ob_.getName() + " Finding Chart " + fcs[currentIndex].getFcName());
        }
            
    }

    public void setStatusString() {
        int length = ob_.getFindingCharts().length;
        statusLabel.setText((currentIndex + 1) + " of " + length);
    }

    private class CancelActionListener extends AbstractAction {

        public CancelActionListener() {
            super("Cancel");
        }

        public void actionPerformed(ActionEvent e) {
            setVisible(false);
        }

    }

    private class PrevActionListener extends AbstractAction {

        public PrevActionListener() {
            super("Previous");
        }

        public void actionPerformed(ActionEvent e) {
            FindingChart[] fcs = ob_.getFindingCharts();
            if (currentIndex > 0)
                currentIndex--;
            else {
                return;
            }
            try {
                displayCurrentFindingChart(fcs[currentIndex]);
            } catch (ObjectNotFoundException e1) {
                ErrorMessages.announceNoObject(comp, e1);
            }
            setPrevNextButton();
        }
    }

    private class NextActionListener extends AbstractAction {

        public NextActionListener() {
            super("Next");
        }

        public void actionPerformed(ActionEvent e) {
            //int length = ob_.getFindingCharts().length;
            FindingChart[] fcs = ob_.getFindingCharts();
            if (currentIndex < fcs.length)
                currentIndex++;
            else {
                return;
            }
            try {
                displayCurrentFindingChart(fcs[currentIndex]);
            } catch (ObjectNotFoundException e1) {
                ErrorMessages.announceNoObject(comp, e1);
            }
            setPrevNextButton();
        }

    }

    private void setPrevNextButton() {
        FindingChart[] fcs = ob_.getFindingCharts();
        if (fcs.length == 1) {
            nextBtn.setEnabled(false);           
            prevBtn.setEnabled(false);
        } else if (currentIndex == 0) {
            prevBtn.setEnabled(false);
            nextBtn.setEnabled(true);
        } else if (currentIndex == fcs.length - 1) {
            prevBtn.setEnabled(true);
            nextBtn.setEnabled(false);
        } else {
            prevBtn.setEnabled(true);
            nextBtn.setEnabled(true);
        }
    }

    /**
         * Sets the object to be displayed by the ObjectView within this
         * DocHolder.  However, setDisplayedObject() may not immediately be
         * called on the ObjectView, if it is not currently visible to the user.
         * 
         * @param   vc          Containing ViewContainer for this DocHolder
         * @param   isVisible   True if window is now visible, false otherwise
         */
    public void setDisplayedObject(ObservationBlock obj) {
        if (ob_ != null) {
            ob_.removePropertyChangeListener(this);
        }
        ob_ = obj;
        if (ob_ != null) {
            ob_.addPropertyChangeListener(this);
        }
    }

    /**
     * @see org.eso.ohs.persistence.ObjectSelectionListener#objectSelected(org.eso.ohs.apps.ObjectSelectionEvent)
     */
    public void objectSelected(ObjectSelectionEvent ev) {
        if (!isVisible())
            return;
        if (ev.getMedia().equals(Media.DBASE))
            return;
        ObjectManager objMgr = ObjectManager.getObjectManager();
        if (ev.getObjectId() == ObjectSelectionEvent.NO_OBJECT) {
            reinit();
            return;
        }
        try {
            StorableObject bo =
                objMgr.getBusObj(
                    ev.getMedia(),
                    ev.getObjectId(),
                    ev.getObjectClass());
            if (bo instanceof ObservationBlock) {
                setDisplayedObject((ObservationBlock) bo);
                init(ob_);
            } else {
               reinit();
            }

        } catch (ObjectNotFoundException e) {
            ErrorMessages.announceIOError(null, e);

        } catch (ObjectIOException e) {
            ErrorMessages.announceIOError(null, e);

        } catch (IOException e) {
            ErrorMessages.announceIOError(null, e);
        }

    }
    public void reinit() {
        setTitle("No OB Selected ");
        repaint();
    }

    /**
     * @see java.beans.PropertyChangeListener#propertyChange(java.beans.PropertyChangeEvent)
     */
    public void propertyChange(PropertyChangeEvent ev) {
        //makeTitle();
        if (ev.getPropertyName() != null
            && ev.getPropertyName().equals("FindingCharts")) {
            try {
                init(ob_);
            } catch (IOException e) {
                ErrorMessages.announceIOError(null, e);
            } catch (ObjectNotFoundException e) {
                ErrorMessages.announceNoObject(null, e);
            } catch (ObjectIOException e) {
                ErrorMessages.announceIOError(null, e);
            }

        }

    }

}
