package org.eso.ohs.phase2.visibility;


/**
 * JNIWrapper - class is the native declaration which provides
 * the bridge to run the native functions in the JVM.  Each
 * function maps onto a SLALIB C functions. SLALIB is a 
 * library of C routines concerned with astronomical 
 *  position and time.
 *
 * @see AirMass
 * @see FLI
 * @see MoonDistance
 * @see SunAngle
 * @see SiderealTime
 * 
 */


public class JNIWrapper {

    public static native double jnislaAirmass(double zd);

    public static native double[] jnislaDe2h( double ha, double dec, 
      double phi);
    
    public static native double jnislaDsep(double a1, double b1, 
      double a2, double b2);

    public static native double jnislaGmsta(double date, double ut);

    public static native double[] jnislaMap( double rm, double dm, 
      double pr, double pd, double px, double rv, double eq, 
      double date);

    public static native double[] jnislaRdplan(double date, int np, 
      double elong, double phi);
    
    public static native double jnislaZenith(double ha, double dec, double phi);


    /* The following are not called at this time
    public static native double[] jnislaDmoon( double date );
    public static native double jnislaLST(double ut);
    public static native double[] jnislaAop ( double rap, double dap, 
      double date, double dut, double elongm, double phim, 
      double hm, double xp, double yp, double tdk, double pmb,
      double rh, double wl, double tlr);
    */

    /**
     * Load library that contains the native code.
     */
    static {
        System.loadLibrary("vplib");
        //System.out.println("Loaded vplib");
    }
    
}
