package org.eso.ohs.phase2.visibility;

import java.util.Date;

import org.eso.ohs.dfs.TimeInterval;

/**
 *  Created on Jul 25, 2005
 * 
 * @author pnunes
 *
 */
public interface VisibilityCalc {
	
	/**
     * getLST get the local side real time
     * Substract the longitude of the Observatory from
     * the returned sidereal time for the local sidereal time
     * of the observatory 
     * @param jdate double Julian Date
     * @param longit double the longitude of the Observatory
     * @return lst double 
     * 
     */
	public double getLST(double jdate, double longitude);
	
	
	/**
     * getFLIValue - get the Fractional Lunar Illumination. 
     * @param jdate double Julian Date
     * @param longit double the longitude of the Observatory
     * @param lat double the latitude of the Observatory
     * @return FLI double 
     */
    public double getFLIValue(double jdate, double longit, double lat);
    
    /**
     * getAirMass of the target to the Observatory
     * @param jdate double Julian Date
     * @param lst double the Local Sidereal Time
     * @param target Ra double
     * @param target Dec double
     * @param lat double the latitude of the Observatory
     * @return Air Mass double
     */
    public double getAirMassValue(
        double jdate,
        double lst,
        double targetRA,
        double targetDEC,
        double lat);
    
    /**
     * getMoonDistValue - get the Moon angle of seperation
     * from the target
     * @param jdate double Julian Date
     * @param target Ra double
     * @param target Dec double
     * @param longit double the longitude of the Observatory
     * @param lat double the latitude of the Observatory
     * @return Moon Distance
     */
    public double getMoonDistValue(
        double jdate,
        double targetRA,
        double targetDEC,
        double longit,
        double lat);
    

    /**
     * @param jdate Julian Date
     * @param longit the longitude of the Observatory
     * @param lat the latitude of the Observatory
     * @return 1  if moon is visible or 0
     */
    public double getMoonVisible(double jdate, double longit, double lat);    

    /**
     * @param jdate Julian Date
     * @param longit the longitude of the Observatory
     * @param lat the latitude of the Observatory
     * @return Moon angle with the horizon
     */
    public double getMoonAngle(double jdate, double longit, double lat);    
    
    /**
     * getSunValue - get the Sun Twilight angle 
     * @param jdate double Julian Date
     * @param longit double the longitude of the Observatory
     * @param lat double the latitude of the Observatory
     * @param lst double the Local Sidereal Time
     * @return sun angle in degrees
     */
    public double getSunValue(
        double jdate,
        double longit,
        double lat,
        double lst);

    
    /**
     * @param jdate
     * @param longit
     * @param lat
     * @return
     */
    public double getMoonRa(double jdate, double longit, double lat); 
    
    
    /**
     * @param jdate
     * @param longit
     * @param lat
     * @return
     */
    public double getMoonDec(double jdate, double longit, double lat);
    
    
    /**
     * getTimeIntervalValue - get the Sidereal Time Interval Value If the time
     * interval has not been defined in the OB any time period is satisfactory
     * set the returned value to 1. Else if the start and end time does not fall
     * within this interval time (precision) set returned value to 0.
     * @param ObservationBlock
     * @param julian Date double
     * @return time interval value
     */
    public double getSiderealTimeIntervalValue(
        TimeInterval [] timeInts,
        Date utDate,
        double jdate); 
    
    
}

